"""
[summary]

[extended_summary]
"""

# region [Imports]

# * Standard Library Imports ------------------------------------------------------------------------------------------------------------------------------------>

import gc
import os
import sys
import unicodedata
import shutil


# * Third Party Imports ----------------------------------------------------------------------------------------------------------------------------------------->

from tqdm import tqdm
from colorama import init as colorama_init, Fore, Back, Style
colorama_init(autoreset=True)
from win10toast import ToastNotifier
# import discord

# import requests

# import pyperclip

# import matplotlib.pyplot as plt

# from bs4 import BeautifulSoup

# from dotenv import load_dotenv

# from discord import Embed, File

# from discord.ext import commands, tasks

# from github import Github, GithubException

# from jinja2 import BaseLoader, Environment

# from natsort import natsorted

# from fuzzywuzzy import fuzz, process


# * PyQt5 Imports ----------------------------------------------------------------------------------------------------------------------------------------------->

# from PyQt5.QtGui import QFont, QIcon, QBrush, QColor, QCursor, QPixmap, QStandardItem, QRegExpValidator

# from PyQt5.QtCore import (Qt, QRect, QSize, QObject, QRegExp, QThread, QMetaObject, QCoreApplication,
#                           QFileSystemWatcher, QPropertyAnimation, QAbstractTableModel, pyqtSlot, pyqtSignal)

# from PyQt5.QtWidgets import (QMenu, QFrame, QLabel, QAction, QDialog, QLayout, QWidget, QWizard, QMenuBar, QSpinBox, QCheckBox, QComboBox, QGroupBox, QLineEdit,
#                              QListView, QCompleter, QStatusBar, QTableView, QTabWidget, QDockWidget, QFileDialog, QFormLayout, QGridLayout, QHBoxLayout,
#                              QHeaderView, QListWidget, QMainWindow, QMessageBox, QPushButton, QSizePolicy, QSpacerItem, QToolButton, QVBoxLayout, QWizardPage,
#                              QApplication, QButtonGroup, QRadioButton, QFontComboBox, QStackedWidget, QListWidgetItem, QSystemTrayIcon, QTreeWidgetItem,
#                              QDialogButtonBox, QAbstractItemView, QCommandLinkButton, QAbstractScrollArea, QGraphicsOpacityEffect, QTreeWidgetItemIterator)


# * Gid Imports ------------------------------------------------------------------------------------------------------------------------------------------------->

import gidlogger as glog
from datetime import datetime

# * Local Imports ----------------------------------------------------------------------------------------------------------------------------------------------->
from antistasi_console_log_parser.items.log_item import LogItem
from antistasi_console_log_parser.items.output_item import OutputItem
from antistasi_console_log_parser.utils.misc import LOG_MSG_PREFIX, LOG_MSG_SUFFIX, pathmaker, is_one_file_exe, create_shortcut
from antistasi_console_log_parser.database.database_manager import storage_db
# endregion[Imports]

# region [TODO]


# endregion [TODO]

# region [AppUserData]


# endregion [AppUserData]

# region [Logging]

log = glog.aux_logger(__name__)


# endregion[Logging]

# region [Constants]
toaster = ToastNotifier()
ICONS_LOCATION = pathmaker(os.path.abspath(os.path.dirname(__file__)), "icons") if is_one_file_exe() is False else pathmaker(sys._MEIPASS, "icons")
TOASTER_ICON = pathmaker(ICONS_LOCATION, "arma_db_icon.ico")

# endregion[Constants]


def update_pb(progress_bar, steps):
    def pb_updater():
        try:
            progress_bar.update(steps)
        except:
            return 0
        return 0
    return pb_updater


def dynamic_terminal_width():
    return os.get_terminal_size().columns


def load_file_to_db(in_file_path):
    log_item = LogItem(in_file_path)
    log_item.parse()
    log_item.to_db()
    return 0 if log_item.for_insert_to_db is False else 1


def load_folder_into_db(in_folder_path):
    log_items = []
    for index, file in enumerate(os.scandir(in_folder_path)):
        if file.is_file() and file.name.endswith('.log'):
            print(f" {Fore.GREEN + Back.BLACK}# {index+1} -- Found Log file '{file.name}'{Style.RESET_ALL} ".ljust(dynamic_terminal_width(), '-'))
            log_items.append(LogItem(file.path))

    print('')
    list(map(lambda x: x.parse(), tqdm(log_items, desc=f"{Fore.WHITE + Back.BLACK}Parsing log Files, this could take some time, please wait!{Style.RESET_ALL}", unit=" log file", colour='green', leave=False)))
    print('')

    with tqdm(total=len(log_items), desc=f"{Fore.WHITE + Back.BLACK}Inserting Data into Database, this could take some time, please wait!{Style.RESET_ALL}", unit=' log file', colour='blue', smoothing=0.1) as pb:
        for log_item in log_items:
            log_item.to_db()
            pb.update(1)
    return len([item for item in log_items if item.for_insert_to_db is True]), len(log_items)


def get_as_excel():
    print(f" {Fore.GREEN + Back.BLACK}Querying Database{Style.RESET_ALL} ".center(dynamic_terminal_width(), '-') + "\n")
    output_item = storage_db.get_steam_id_and_player_names()
    print(f" {Fore.GREEN + Back.BLACK}Writing Excel-File to '{output_item.default_file_name}'{Style.RESET_ALL} ".center(dynamic_terminal_width(), '-') + "\n")
    output_item.to_excel()


def copy_icon():
    new_path = pathmaker(os.getenv('APP_DATA_FOLDER'), 'bertha_icon.ico')
    if os.path.isfile(new_path) is False:
        with open(pathmaker(ICONS_LOCATION, 'bertha_icon.ico'), 'rb') as f:
            with open(new_path, 'wb') as fw:
                fw.write(f.read())
    os.environ['SHORTCUT_ICON'] = new_path


def create_extras():
    copy_icon()
    create_shortcut(os.getenv('APP_NAME').title(), sys.argv[0])


def main():

    inserted_log_files = 0
    total_log_files = 0
    arguments = sys.argv[1:]

    if arguments in [None, [], ""]:
        print(f" {Fore.GREEN + Back.BLACK}Writing collected Data to '{OutputItem.default_file_name}'{Style.RESET_ALL} ".center(dynamic_terminal_width(), '-') + "\n")
        create_extras()
        get_as_excel()

    elif list(map(lambda x: x.casefold(), arguments)) == ['clear']:
        appdata_path = pathmaker(os.getenv("APP_DATA_FOLDER"))
        input_str = input("Do you really want to delete the appdata folder with the Database for this application? This cannot be reversed and the deleted files will not go to the Recycling Bin!\n[Y/N]")

        if input_str.casefold() in ['y', 'yes']:
            if os.path.basename(os.path.dirname(appdata_path)).casefold() == os.getenv('AUTHOR_NAME').casefold():
                start_path = os.path.dirname(appdata_path)
            else:
                start_path = appdata_path
            shutil.rmtree(start_path)
            print(f'appdata folder ("{start_path}") for this application with the Database was deleted!')
            return
        if input_str.casefold() in ['n', 'no', 'cancel', 'abort']:
            print('User cancellation!')
            return
        else:
            print('invalid Input, cancelling!')
            return
    else:
        create_extras()
        for path in arguments:
            if os.path.exists(path) is False:
                print(f" {Fore.RED + Back.BLACK}Input needs to be an existing folder path or file path!{Style.RESET_ALL} ".center(dynamic_terminal_width(), "!") + '\n')
                print(f" '{Fore.BLUE + Back.WHITE}{path}{Style.RESET_ALL}' ".center(dynamic_terminal_width(), "?"))
                print(f" {Fore.RED + Back.BLACK} is either not a path or the file/folder does not exist!{Style.RESET_ALL} ".center(dynamic_terminal_width(), "!") + '\n')
                raise RuntimeError(f"'{path}' is not a valid input!")
            if os.path.isdir(path) is True:
                print(f" {Fore.GREEN + Back.BLACK}Loading Folder '{os.path.basename(path)}' into Database{Style.RESET_ALL} ".center(dynamic_terminal_width(), '-'))
                insert_data = load_folder_into_db(path)
                inserted_log_files += insert_data[0]
                total_log_files += insert_data[1]
            elif os.path.isfile(path) is True:
                print(f" {Fore.GREEN + Back.BLACK}Parsing file '{os.path.basename(path)}' to Database{Style.RESET_ALL} ".center(dynamic_terminal_width(), '-'))
                inserted_log_files += load_file_to_db(path)
                total_log_files += 1
        toaster.show_toast(title="Antistasi_Console_Log_Parser finished!",
                           msg=f"Inserted {inserted_log_files} log files into the Database, out of {total_log_files} provided log files",
                           duration=5,
                           threaded=True,
                           icon_path=TOASTER_ICON)
    print("\n\n" + f" {Fore.GREEN + Back.BLACK}Done!{Style.RESET_ALL} ".center(dynamic_terminal_width(), '-') + "\n")

# region[Main_Exec]


if __name__ == '__main__':
    main()
# endregion[Main_Exec]
