"""
[summary]

[extended_summary]
"""

# region [Imports]

# * Standard Library Imports ------------------------------------------------------------------------------------------------------------------------------------>

import gc
import os
import sys
from typing import List

from datetime import datetime, time, timedelta, timezone, tzinfo
import sqlite3 as sqlite
# * Third Party Imports ----------------------------------------------------------------------------------------------------------------------------------------->

# import discord

# import requests

# import pyperclip

# import matplotlib.pyplot as plt

# from bs4 import BeautifulSoup

# from dotenv import load_dotenv

# from discord import Embed, File

# from discord.ext import commands, tasks

# from github import Github, GithubException

# from jinja2 import BaseLoader, Environment

# from natsort import natsorted

# from fuzzywuzzy import fuzz, process


# * PyQt5 Imports ----------------------------------------------------------------------------------------------------------------------------------------------->

# from PyQt5.QtGui import QFont, QIcon, QBrush, QColor, QCursor, QPixmap, QStandardItem, QRegExpValidator

# from PyQt5.QtCore import (Qt, QRect, QSize, QObject, QRegExp, QThread, QMetaObject, QCoreApplication,
#                           QFileSystemWatcher, QPropertyAnimation, QAbstractTableModel, pyqtSlot, pyqtSignal)

# from PyQt5.QtWidgets import (QMenu, QFrame, QLabel, QAction, QDialog, QLayout, QWidget, QWizard, QMenuBar, QSpinBox, QCheckBox, QComboBox, QGroupBox, QLineEdit,
#                              QListView, QCompleter, QStatusBar, QTableView, QTabWidget, QDockWidget, QFileDialog, QFormLayout, QGridLayout, QHBoxLayout,
#                              QHeaderView, QListWidget, QMainWindow, QMessageBox, QPushButton, QSizePolicy, QSpacerItem, QToolButton, QVBoxLayout, QWizardPage,
#                              QApplication, QButtonGroup, QRadioButton, QFontComboBox, QStackedWidget, QListWidgetItem, QSystemTrayIcon, QTreeWidgetItem,
#                              QDialogButtonBox, QAbstractItemView, QCommandLinkButton, QAbstractScrollArea, QGraphicsOpacityEffect, QTreeWidgetItemIterator)


# * Gid Imports ------------------------------------------------------------------------------------------------------------------------------------------------->

import gidlogger as glog


# * Local Imports ----------------------------------------------------------------------------------------------------------------------------------------------->

from antistasi_console_log_parser.utils.gidsql.facade import GidSqliteDatabase
from antistasi_console_log_parser.utils.misc import pathmaker, is_one_file_exe
from antistasi_console_log_parser.items.output_item import OutputItem
# endregion[Imports]

# region [TODO]


# endregion [TODO]

# region [AppUserData]


# endregion [AppUserData]

# region [Logging]

log = glog.aux_logger(__name__)

# endregion[Logging]

# region [Constants]


SCRIPT_LOCATION = pathmaker(os.path.abspath(os.path.dirname(__file__)), "sql_scripts") if is_one_file_exe() is False else pathmaker(sys._MEIPASS, "sql_scripts")


# endregion[Constants]

def adapt_time(in_time: datetime.time):
    return in_time.isoformat()


sqlite.register_adapter(time, adapt_time)


class DatabaseManager:

    def __init__(self):
        self.db = GidSqliteDatabase(db_location=pathmaker(os.getenv("APP_DATA_FOLDER"), 'Storage.db'), script_location=SCRIPT_LOCATION, log_execution=False)
        self.db.pragmas = ["PRAGMA cache_size(-250000)", "PRAGMA synchronous(OFF)"]
        self.was_created = self.db.startup_db()
        self.db.vacuum()

    def insert_player_data(self, data: List[dict]):
        self.db.write('insert_steam_id', [(item.get('id'), item.get('id')) for item in data])
        self.db.write('insert_player_name', [(item.get('name'), item.get('name')) for item in data])
        self.db.write('insert_player_name_steam_id_reference', [(item.get('id'), item.get('name')) for item in data])
        self.db.write('insert_login_steam_id', [(item.get('time'), item.get('id')) for item in data])
        self.db.write('insert_login_player_name', [(item.get('time'), item.get('name')) for item in data])

    def get_steam_id_and_player_names(self):
        output_item = OutputItem()
        result = self.db.query('get_steam_id_and_player_names', row_factory=True)
        for row in result:
            steam_id = row['steam_id']
            name = row['name']
            amount_logins = self.db.query("get_max_logins_player_name", (name,), fetch=self.db.One)[0]
            output_item.add_data(steam_id, (name, amount_logins))
        login_result = self.db.query("get_all_logins", row_factory=True)
        for row in login_result:
            output_item.add_login_count(row['steam_id'])

        return output_item


storage_db = DatabaseManager()

# region[Main_Exec]
if __name__ == '__main__':
    pass

# endregion[Main_Exec]
