"""
[summary]

[extended_summary]
"""

# region [Imports]

# * Standard Library Imports ------------------------------------------------------------------------------------------------------------------------------------>


import os
import unicodedata
from datetime import datetime, timedelta, timezone, time, tzinfo

# * Third Party Imports ----------------------------------------------------------------------------------------------------------------------------------------->

# import discord

# import requests

# import pyperclip

# import matplotlib.pyplot as plt

# from bs4 import BeautifulSoup

# from dotenv import load_dotenv

# from discord import Embed, File

# from discord.ext import commands, tasks

# from github import Github, GithubException

# from jinja2 import BaseLoader, Environment

# from natsort import natsorted

# from fuzzywuzzy import fuzz, process


# * Gid Imports ------------------------------------------------------------------------------------------------------------------------------------------------->

import gidlogger as glog


# * Local Imports ----------------------------------------------------------------------------------------------------------------------------------------------->
from hashlib import blake2b
import pandas as pd
from antistasi_console_log_parser.data.regexes import CONNECT_REGEX
from antistasi_console_log_parser.database.database_manager import storage_db
from antistasi_console_log_parser.utils.misc import pathmaker
from antistasi_console_log_parser.utils.file_hasher import FileHasher
# endregion[Imports]

# region [TODO]


# endregion [TODO]

# region [AppUserData]


# endregion [AppUserData]

# region [Logging]

log = glog.aux_logger(__name__)


# endregion[Logging]

# region [Constants]

# endregion[Constants]


class LogItem:
    connect_regex = CONNECT_REGEX
    db = storage_db
    exclude_names = ['headlessclient', 'hc', "headlessclient (2)", 'hc_1', 'hc_2', 'hc_0', 'HC1', 'HC0', 'HC2']
    hasher = FileHasher()

    def __init__(self, log_file_path):
        self.path = pathmaker(log_file_path)
        self.data = []
        self.log_file_hash = self.get_log_file_hash()
        self.for_insert_to_db = True

    def get_log_file_hash(self):

        with open(self.path, 'rb') as f:
            content = f.read()
            content += os.path.basename(self.path).encode('utf-8', errors='ignore')
            content += str(os.stat(self.path).st_ctime).encode('utf-8', errors='ignore')
            return blake2b(content).hexdigest()

    def generate_lines(self):
        with open(self.path, 'r', encoding='utf-8', errors='ignore') as f:
            line = next(f, 'end of Generator').strip('\n')
            while line not in ['class Session', 'end of Generator']:
                yield line
                line = next(f, 'end of Generator').strip('\n')

    def parse(self):
        if self.log_file_hash in self.hasher:
            self.for_insert_to_db = False
            return
        for line in self.generate_lines():
            line_match = self.connect_regex.search(line)
            if line_match:

                connect_time = time(tzinfo=timezone.utc, **{key: int(value) for key, value in line_match.groupdict().items() if key in ['hour', 'minute', 'second']})

                player_name = line_match.group('player_name')
                steam_id = line_match.group('steam_id')
                if player_name not in self.exclude_names:
                    try:
                        self.data.append({'time': connect_time, 'name': player_name, 'id': int(steam_id)})
                    except ValueError as error:
                        print(f"{player_name=}")
                        raise error
        self.hasher.append(self.log_file_hash)

    def to_db(self):
        if self.for_insert_to_db is True:
            self.db.insert_player_data(self.data)


# region[Main_Exec]
if __name__ == '__main__':
    pass

# endregion[Main_Exec]
